"use strict";

/* eslint-disable no-console */

import React, { memo, useCallback, useEffect, useState } from 'react';
import { ActivityIndicator, ScrollView, StyleSheet, Text, View } from 'react-native';
import { getStylesFromShikiTheme } from '../utils/style';
import { useShiki } from './SyntaxContext';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const DEBUG = true;
function log(...args) {
  if (DEBUG) {
    console.log('[SyntaxHighlighter]', ...args);
  }
}
const styles = StyleSheet.create({
  container: {
    flex: 1
  },
  content: {
    padding: 10
  },
  line: {
    flexDirection: 'row'
  },
  lineContent: {
    flexDirection: 'row',
    flexWrap: 'wrap'
  },
  lineNumber: {
    width: 40,
    paddingRight: 8,
    opacity: 0.5
  },
  lineNumberText: {
    textAlign: 'right'
  },
  error: {
    color: '#ff6b6b'
  },
  bold: {
    fontWeight: 'bold'
  },
  italic: {
    fontStyle: 'italic'
  },
  underline: {
    textDecorationLine: 'underline'
  }
});
export const SyntaxHighlighter = /*#__PURE__*/memo(({
  code,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  showLineNumbers = false,
  renderLineNumber,
  loadingComponent,
  errorComponent
}) => {
  const {
    highlighter,
    isReady
  } = useShiki();
  const [tokens, setTokens] = useState([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);
  const stylesheet = React.useMemo(() => {
    const styles = getStylesFromShikiTheme(theme);
    log('Generated stylesheet from theme:', {
      themeName: typeof theme === 'string' ? theme : theme.name,
      themeSettings: theme.settings?.length,
      generatedStyles: Object.keys(styles),
      rootStyle: styles.root
    });
    return styles;
  }, [theme]);
  const highlight = useCallback(async () => {
    if (!highlighter || !isReady) {
      log('Highlighter not ready:', {
        highlighter: !!highlighter,
        isReady
      });
      return;
    }
    try {
      setLoading(true);
      setError(null);
      const normalizedLang = language.toLowerCase();
      const themeName = typeof theme === 'string' ? theme : theme.name;
      log('Highlighting code:', {
        language: normalizedLang,
        theme: themeName,
        codeLength: code.length
      });

      // First load the theme if it's not already loaded
      await highlighter.loadTheme(theme);

      // Then tokenize with explicit options
      const result = highlighter.codeToTokensBase(code, {
        lang: normalizedLang,
        theme: themeName,
        includeExplanation: true // Add this to get more token information
      });

      // Add more detailed logging
      log('Tokenization result:', {
        lines: result.length,
        firstLineTokens: result[0]?.length,
        sampleTokens: result[0]?.map(token => ({
          content: token.content,
          color: token.color,
          fontStyle: token.fontStyle,
          explanation: token.explanation?.[0]?.scopes
        }))
      });
      setTokens(result);
    } catch (err) {
      const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code';
      log('Highlighting error:', err);
      setError(errorMessage);
    } finally {
      setLoading(false);
    }
  }, [code, language, theme, highlighter, isReady]);
  useEffect(() => {
    highlight();
  }, [highlight]);
  const renderToken = useCallback((token, lineIndex, tokenIndex) => {
    // Get theme styles for this token's explanation scopes
    const themeStyles = token.explanation?.reduce((acc, exp) => {
      exp.scopes.forEach(scope => {
        if (stylesheet[scope.scopeName]) {
          Object.assign(acc, stylesheet[scope.scopeName]);
        }
      });
      return acc;
    }, {}) || {};

    // Combine all applicable styles
    const tokenStyle = StyleSheet.flatten([textStyle, {
      fontSize
    },
    // Apply theme-based styles from token explanations
    themeStyles,
    // Apply direct token color (takes precedence over theme styles)
    token.color ? {
      color: token.color
    } : null,
    // Apply font styles
    token.fontStyle === 1 && styles.italic, token.fontStyle === 2 && styles.bold, token.fontStyle === 4 && styles.underline]);
    if (DEBUG && lineIndex === 0 && tokenIndex === 0) {
      log('Rendering first token:', {
        content: token.content,
        color: token.color,
        scopes: token.explanation?.map(exp => exp.scopes.map(s => s.scopeName)),
        themeStyles,
        fontStyle: token.fontStyle,
        finalStyle: tokenStyle
      });
    }
    return /*#__PURE__*/_jsx(Text, {
      style: tokenStyle,
      selectable: false,
      children: token.content
    }, `${lineIndex}-${tokenIndex}`);
  }, [fontSize, textStyle, stylesheet]);
  const renderLine = useCallback((lineTokens, lineIndex) => {
    if (DEBUG && lineIndex === 0) {
      log('Rendering first line:', {
        tokenCount: lineTokens.length,
        tokens: lineTokens.map(t => ({
          content: t.content,
          color: t.color
        }))
      });
    }
    return /*#__PURE__*/_jsxs(View, {
      style: styles.line,
      children: [showLineNumbers && /*#__PURE__*/_jsx(View, {
        style: styles.lineNumber,
        children: renderLineNumber ? renderLineNumber(lineIndex + 1) : /*#__PURE__*/_jsx(Text, {
          style: styles.lineNumberText,
          children: (lineIndex + 1).toString().padStart(3, ' ')
        })
      }), /*#__PURE__*/_jsx(View, {
        style: styles.lineContent,
        children: lineTokens.map((token, tokenIndex) => renderToken(token, lineIndex, tokenIndex))
      })]
    }, lineIndex);
  }, [renderToken, showLineNumbers, renderLineNumber]);
  useEffect(() => {
    log('Component mounted with props:', {
      language,
      theme: typeof theme === 'string' ? theme : theme.name,
      isReady,
      hasHighlighter: !!highlighter
    });
    return () => log('Component unmounted');
  }, [language, theme, isReady, highlighter]);
  if (loading) {
    return loadingComponent || /*#__PURE__*/_jsx(View, {
      style: [styles.container, {
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsx(ActivityIndicator, {})
    });
  }
  if (error) {
    return errorComponent?.(error) || /*#__PURE__*/_jsx(View, {
      style: [styles.container, {
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsxs(Text, {
        style: styles.error,
        children: ["Error:", error]
      })
    });
  }
  const combinedStyle = [styles.container, stylesheet.root, scrollViewProps?.style];
  log('Rendering with styles:', {
    containerStyle: combinedStyle,
    tokenCount: tokens.reduce((acc, line) => acc + line.length, 0)
  });
  return /*#__PURE__*/_jsx(ScrollView, {
    ...scrollViewProps,
    style: combinedStyle,
    contentContainerStyle: [styles.content, containerStyle],
    horizontal: true,
    children: /*#__PURE__*/_jsx(View, {
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
});
//# sourceMappingURL=SyntaxHighlighter.js.map