/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import React, { memo, useCallback, useEffect, useState } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, StyleSheet, Text, type TextStyle, View, type ViewStyle } from 'react-native'
import { getStylesFromShikiTheme } from '../utils/style'
import { useShiki } from './SyntaxContext'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG) {
    console.log('[SyntaxHighlighter]', ...args)
  }
}

interface SyntaxHighlighterProps {
  code: string
  language: string
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  showLineNumbers?: boolean
  renderLineNumber?: (lineNumber: number) => React.ReactNode
  loadingComponent?: React.ReactNode
  errorComponent?: (error: string) => React.ReactNode
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
  },
  content: {
    padding: 10,
  },
  line: {
    flexDirection: 'row',
  },
  lineContent: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
  lineNumber: {
    width: 40,
    paddingRight: 8,
    opacity: 0.5,
  },
  lineNumberText: {
    textAlign: 'right',
  },
  error: {
    color: '#ff6b6b',
  },
  bold: {
    fontWeight: 'bold',
  },
  italic: {
    fontStyle: 'italic',
  },
  underline: {
    textDecorationLine: 'underline',
  },
})

export const SyntaxHighlighter = memo(
  ({ code, language, theme, fontSize = 14, textStyle, scrollViewProps, containerStyle, showLineNumbers = false, renderLineNumber, loadingComponent, errorComponent }: SyntaxHighlighterProps) => {
    const { highlighter, isReady } = useShiki()
    const [tokens, setTokens] = useState<ThemedToken[][]>([])
    const [loading, setLoading] = useState(true)
    const [error, setError] = useState<string | null>(null)

    const stylesheet = React.useMemo(() => {
      const styles = getStylesFromShikiTheme(theme)
      log('Generated stylesheet from theme:', {
        themeName: typeof theme === 'string' ? theme : theme.name,
        themeSettings: theme.settings?.length,
        generatedStyles: Object.keys(styles),
        rootStyle: styles.root,
      })
      return styles
    }, [theme])

    const highlight = useCallback(async () => {
      if (!highlighter || !isReady) {
        log('Highlighter not ready:', { highlighter: !!highlighter, isReady })
        return
      }

      try {
        setLoading(true)
        setError(null)

        const normalizedLang = language.toLowerCase()
        const themeName = typeof theme === 'string' ? theme : theme.name

        log('Highlighting code:', {
          language: normalizedLang,
          theme: themeName,
          codeLength: code.length,
        })

        // First load the theme if it's not already loaded
        await highlighter.loadTheme(theme)

        // Then tokenize with explicit options
        const result = highlighter.codeToTokensBase(code, {
          lang: normalizedLang,
          theme: themeName,
          includeExplanation: true, // Add this to get more token information
        })

        // Add more detailed logging
        log('Tokenization result:', {
          lines: result.length,
          firstLineTokens: result[0]?.length,
          sampleTokens: result[0]?.map(token => ({
            content: token.content,
            color: token.color,
            fontStyle: token.fontStyle,
            explanation: token.explanation?.[0]?.scopes,
          })),
        })

        setTokens(result)
      }
      catch (err) {
        const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code'
        log('Highlighting error:', err)
        setError(errorMessage)
      }
      finally {
        setLoading(false)
      }
    }, [code, language, theme, highlighter, isReady])

    useEffect(() => {
      highlight()
    }, [highlight])

    const renderToken = useCallback(
      (token: ThemedToken, lineIndex: number, tokenIndex: number) => {
        // Get theme styles for this token's explanation scopes
        const themeStyles
          = token.explanation?.reduce((acc, exp) => {
            exp.scopes.forEach((scope) => {
              if (stylesheet[scope.scopeName]) {
                Object.assign(acc, stylesheet[scope.scopeName])
              }
            })
            return acc
          }, {} as TextStyle) || {}

        // Combine all applicable styles
        const tokenStyle = StyleSheet.flatten([
          textStyle,
          { fontSize },
          // Apply theme-based styles from token explanations
          themeStyles,
          // Apply direct token color (takes precedence over theme styles)
          token.color ? { color: token.color } : null,
          // Apply font styles
          token.fontStyle === 1 && styles.italic,
          token.fontStyle === 2 && styles.bold,
          token.fontStyle === 4 && styles.underline,
        ])

        if (DEBUG && lineIndex === 0 && tokenIndex === 0) {
          log('Rendering first token:', {
            content: token.content,
            color: token.color,
            scopes: token.explanation?.map(exp => exp.scopes.map(s => s.scopeName)),
            themeStyles,
            fontStyle: token.fontStyle,
            finalStyle: tokenStyle,
          })
        }

        return (
          <Text key={`${lineIndex}-${tokenIndex}`} style={tokenStyle} selectable={false}>
            {token.content}
          </Text>
        )
      },
      [fontSize, textStyle, stylesheet],
    )

    const renderLine = useCallback(
      (lineTokens: ThemedToken[], lineIndex: number) => {
        if (DEBUG && lineIndex === 0) {
          log('Rendering first line:', {
            tokenCount: lineTokens.length,
            tokens: lineTokens.map(t => ({ content: t.content, color: t.color })),
          })
        }

        return (
          <View key={lineIndex} style={styles.line}>
            {showLineNumbers && (
              <View style={styles.lineNumber}>{renderLineNumber ? renderLineNumber(lineIndex + 1) : <Text style={styles.lineNumberText}>{(lineIndex + 1).toString().padStart(3, ' ')}</Text>}</View>
            )}
            <View style={styles.lineContent}>{lineTokens.map((token, tokenIndex) => renderToken(token, lineIndex, tokenIndex))}</View>
          </View>
        )
      },
      [renderToken, showLineNumbers, renderLineNumber],
    )

    useEffect(() => {
      log('Component mounted with props:', {
        language,
        theme: typeof theme === 'string' ? theme : theme.name,
        isReady,
        hasHighlighter: !!highlighter,
      })
      return () => log('Component unmounted')
    }, [language, theme, isReady, highlighter])

    if (loading) {
      return (
        loadingComponent || (
          <View style={[styles.container, { padding: 10 }, containerStyle]}>
            <ActivityIndicator />
          </View>
        )
      )
    }

    if (error) {
      return (
        errorComponent?.(error) || (
          <View style={[styles.container, { padding: 10 }, containerStyle]}>
            <Text style={styles.error}>
              Error:
              {error}
            </Text>
          </View>
        )
      )
    }

    const combinedStyle = [styles.container, stylesheet.root, scrollViewProps?.style]
    log('Rendering with styles:', {
      containerStyle: combinedStyle,
      tokenCount: tokens.reduce((acc, line) => acc + line.length, 0),
    })

    return (
      <ScrollView {...scrollViewProps} style={combinedStyle} contentContainerStyle={[styles.content, containerStyle]} horizontal>
        <View>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
      </ScrollView>
    )
  },
)

export type { SyntaxHighlighterProps }
