/* eslint-disable no-console */
import type { ThemeRegistrationAny } from '@shikijs/types'
import type { TextStyle } from 'react-native'
import transform, { type StyleTuple } from 'css-to-react-native'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG) {
    console.log('[StyleUtils]', ...args)
  }
}

export interface ShikiStyleSheet {
  [key: string]: TextStyle
}

const ALLOWED_STYLE_PROPERTIES: Record<string, boolean> = {
  color: true,
  background: true,
  backgroundColor: true,
  fontWeight: true,
  fontStyle: true,
  textDecoration: true,
}

export function getStylesFromShikiTheme(theme: ThemeRegistrationAny): ShikiStyleSheet {
  log('Processing theme:', {
    name: typeof theme === 'string' ? theme : theme.name,
    hasSettings: !!theme.settings,
    settingsCount: theme.settings?.length,
    hasTokenColors: !!(theme as any).tokenColors?.length,
  })

  const styles: ShikiStyleSheet = {
    root: {
      backgroundColor: theme.bg || theme.colors?.['editor.background'] || '#000000',
      color: theme.fg || theme.colors?.['editor.foreground'] || '#ffffff',
    },
  }

  // Process token colors from the theme
  const tokenColors = (theme as any).tokenColors || theme.settings || []

  tokenColors.forEach((token: any) => {
    if (!token.scope || !token.settings)
      return

    const scopes = Array.isArray(token.scope) ? token.scope : typeof token.scope === 'string' ? token.scope.split(',').map((s: string) => s.trim()) : []

    const style: TextStyle = {}

    // Process color
    if (token.settings.foreground) {
      style.color = token.settings.foreground
    }

    // Process font styles
    if (token.settings.fontStyle) {
      const fontStyles = token.settings.fontStyle.split(' ')
      if (fontStyles.includes('bold')) {
        style.fontWeight = 'bold'
      }
      if (fontStyles.includes('italic')) {
        style.fontStyle = 'italic'
      }
      if (fontStyles.includes('underline')) {
        style.textDecorationLine = 'underline'
      }
    }

    // Apply styles to each scope
    scopes.forEach((scope: string) => {
      if (!scope)
        return
      // Store styles for each scope
      styles[scope] = {
        ...styles[scope],
        ...style,
      }
    })
  })

  log('Generated styles:', {
    rootStyle: styles.root,
    scopeCount: Object.keys(styles).length,
    sampleScopes: Object.keys(styles).slice(0, 5),
    sampleStyles: Object.entries(styles)
      .slice(0, 3)
      .map(([scope, style]) => ({
        scope,
        style,
      })),
  })

  return styles
}

export function cleanStyle(style: Record<string, any>): TextStyle {
  const filteredEntries = Object.entries(style)
    .filter(([key, value]) => ALLOWED_STYLE_PROPERTIES[key] && value !== undefined)
    .map<StyleTuple>(([key, value]) => [key, value])

  if (DEBUG) {
    log('Cleaning style:', {
      input: style,
      filtered: filteredEntries,
    })
  }

  return transform(filteredEntries)
}
